"""Decorator helper functions for logging"""
import functools
from types import FunctionType
from typing import Any, Dict, List
from loguru import logger
from es.elastic.api import connect
from es.baseapi import BaseConnection
from elasticsearch import Elasticsearch, AsyncElasticsearch
from ..constants.config import BasicConfig, BoolConfig, StringConfig
from ..constants.keys import LoggerKeys
from ..helpers.formats import format_elk_url


def check_debug_mode(func):
    """
    Check if debug value is enabled or not.
    If disabled, inner function will be skipped.
    Use for Sync and class methods.
    """

    @functools.wraps(func)
    def wrapper(self, *args, **kwargs):
        try:
            if self.debug or kwargs.get(LoggerKeys.DEBUG.value):
                return
        except AttributeError:
            if kwargs.get(LoggerKeys.DEBUG.value):
                return
        num_of_decorators = kwargs.get(StringConfig.NUM_OF_DECORATORS.value, 0) + 2
        return func(self, *args, num_of_decorators=num_of_decorators, **kwargs)

    return wrapper


async def async_check_debug_mode(func):
    """
    Check if debug value is enabled or not.
    If disabled, inner function will be skipped.
    Use for Async and class methods.
    """

    @functools.wraps(func)
    async def wrapper(self, *args, **kwargs):
        try:
            if self.debug or kwargs.get(LoggerKeys.DEBUG.value):
                return
        except AttributeError:
            if kwargs.get(LoggerKeys.DEBUG.value):
                return
        num_of_decorators = kwargs.get(StringConfig.NUM_OF_DECORATORS.value, 0) + 2
        return await func(self, *args, num_of_decorators=num_of_decorators, **kwargs)

    return wrapper


def print_retry_exception_msg(
    e: Any,
    retry_attempt: int,
    func: FunctionType,
    args: List[Any],
    kwargs: Dict[str, Any],
):
    """Print log error when retry failed"""
    logger.exception(
        "Exception has occurred with the following "
        + f"message: '{e}' "
        + "It was ignored and has occurred for "
        + f"'{retry_attempt}' times. "
        + f"Number of retries left: {BasicConfig.MAX_RETRIES.value - retry_attempt}."
        + "When attempting to run "
        + f"{repr(func)} with the following arguments: "
        + f"{args} keyword arguments: {kwargs}."
    )


def enable_retries(func):
    """
    Allow retries when the inner function fails.
    Use for Sync and class methods.
    """

    @functools.wraps(func)
    def wrapper(self, *args, **kwargs):
        num_of_decorators = kwargs.get(StringConfig.NUM_OF_DECORATORS.value, 0) + 2
        for retry_attempt in range(BasicConfig.MAX_RETRIES.value):
            try:
                return func(self, *args, num_of_decorators=num_of_decorators, **kwargs)
            except Exception as e:
                print_retry_exception_msg(e, retry_attempt + 1, func, args, kwargs)

    return wrapper


async def async_enable_retries(func):
    """
    Allow retries when the inner function fails.
    Use for Async and class methods.
    """

    @functools.wraps(func)
    async def wrapper(self, *args, **kwargs):
        num_of_decorators = kwargs.get(StringConfig.NUM_OF_DECORATORS.value, 0) + 2
        for retry_attempt in range(BasicConfig.MAX_RETRIES.value):
            try:
                return await func(
                    self, *args, num_of_decorators=num_of_decorators, **kwargs
                )
            except Exception as e:
                print_retry_exception_msg(e, retry_attempt + 1, func, args, kwargs)

    return wrapper


def connect_esdb(func):
    """
    Connect to elasticsearch_dbapi library and then closes the connection.
    Use Sync and class methods.
    """

    @functools.wraps(func)
    def wrapper(self, *args, **kwargs):
        conn: BaseConnection = connect(
            host=format_elk_url(self.config, BoolConfig.USE_ES_DB.value),
            port=self.config.get(LoggerKeys.PORT.value) or BasicConfig.PORT.value,
            scheme=StringConfig.SCHEME.value,
            user=self.username,
            password=self.pw,
            verify_certs=BoolConfig.VERIFY_CERTS.value,
        )
        data: Any = func(self, *args, **kwargs)
        conn.close()
        return data

    return wrapper


def connect_elk(func):
    """
    Connect to Sync elasticsearch library.
    Use for Sync and class methods.
    """

    @functools.wraps(func)
    def wrapper(self, *args, **kwargs):
        result = func(self, *args, **kwargs)
        for retry_attempt in range(BasicConfig.MAX_RETRIES.value):
            try:
                self.es = Elasticsearch(
                    [format_elk_url(kwargs)],
                    use_ssl=BoolConfig.USE_SSL.value,
                    http_auth=(self.username, self.pw),
                    verify_certs=BoolConfig.VERIFY_CERTS.value,
                )
                break
            except Exception as e:
                print_retry_exception_msg(e, retry_attempt + 1, func, args, kwargs)
        return result

    return wrapper


def connect_async_elk(func):
    """
    Connect to Async elasticsearch library.
    Use for Async and class methods.
    """

    @functools.wraps(func)
    def wrapper(self, *args, **kwargs):
        result = func(self, *args, **kwargs)
        for retry_attempt in range(BasicConfig.MAX_RETRIES.value):
            try:
                self.es = AsyncElasticsearch(
                    [format_elk_url(kwargs)],
                    use_ssl=BoolConfig.USE_SSL.value,
                    http_auth=(self.username, self.pw),
                    verify_certs=BoolConfig.VERIFY_CERTS.value,
                    max_retries=BasicConfig.MAX_RETRIES.value,
                    retry_on_timeout=BoolConfig.RETRY_ON_TIMEOUT.value,
                )
                break
            except Exception as e:
                print_retry_exception_msg(e, retry_attempt + 1, func, args, kwargs)
        return result

    return wrapper
