"""Logging interface to enforce all logger wrappers to follow the same format"""
from typing import Any, Dict, Optional, Union
import abc
import pandas as pd
from pandas.core.api import DataFrame
from ..constants.config import StringConfig
from ..constants.keys import LoggerKeys
from ..helpers.decorators import check_debug_mode, connect_esdb, enable_retries


class LoggerInterface(abc.ABC):
    """
    The loggers interface used for all logger types.
    """

    def __init__(self, **kwargs) -> None:
        # Required keys
        self.appname: str = kwargs[LoggerKeys.APPNAME.value]

        # Optional keys
        self.debug: bool = (
            True
            if kwargs.get(LoggerKeys.DEBUG.value) is None
            else kwargs.get(LoggerKeys.DEBUG.value)
        )
        self.version: str = (
            kwargs.get(LoggerKeys.VERSION.value) or StringConfig.VERSION.value
        )

        # elasticsearch keys
        self.config: Dict[str, Any] = kwargs
        if not self.debug:
            self.host: str = kwargs[LoggerKeys.HOST.value]
            self.index: str = kwargs[LoggerKeys.INDEX.value]
            self.username: str = kwargs[LoggerKeys.USERNAME.value]
            self.pw: Any = kwargs[LoggerKeys.PW.value]

    @abc.abstractmethod
    def log(
        self,
        level: str,
        logdata: Union[str, Dict[str, Any]],
        custom_func_name: Optional[str] = None,
        use_custom_logdata: Optional[bool] = None,
        **kwargs
    ) -> None:
        """
        Log data to the appropriate loggers.
        """

    @abc.abstractmethod
    async def async_log(
        self,
        level: str,
        logdata: Union[str, Dict[str, Any]],
        custom_func_name: Optional[str] = None,
        use_custom_logdata: Optional[bool] = None,
        **kwargs
    ) -> None:
        """
        Log data to the appropriate loggers.
        """

    @abc.abstractmethod
    def query(
        self, custompayload: Optional[Dict[str, Any]], size: int, cache: bool
    ) -> Any:
        """
        Make query to elasticsearch.
        """

    @abc.abstractmethod
    async def async_query(
        self, custompayload: Optional[Dict[str, Any]], size: int, cache: bool
    ) -> Any:
        """
        Make query to elasticsearch.
        """

    @check_debug_mode
    @connect_esdb
    @enable_retries
    def sql_query(self, query: str, **kwargs) -> DataFrame:
        """
        Make query to elasticsearch-dbapi
        """
        return pd.read_sql(query, kwargs.get("conn"))
